from ovos_config.config import Configuration

from ovos_backend_client.backends.base import BackendType
from ovos_backend_client.backends.offline import OfflineBackend
from ovos_backend_client.backends.ovos import OVOS_API_URL, OVOSAPIBackend
from ovos_backend_client.backends.personal import PersonalBackend
from ovos_backend_client.backends.selene import SELENE_API_URL, SeleneBackend

API_REGISTRY = {
    BackendType.OFFLINE: {
        "admin": True,  # updates mycroft.conf if used
        "device": True,  # shared database with local backend for UI compat
        "dataset": True,  # shared database with local backend for ww tagger UI compat
        "metrics": True,  # shared database with local backend for metrics UI compat
        "wolfram": True,  # key needs to be set
        "geolocate": True,  # nominatim - no key needed
        "stt": True,  # uses OPM and reads from mycroft.conf
        "owm": True,  # key needs to be set
        "email": True,  # smtp config needs to be set
        "oauth": True  # use local backend UI on same device to register apps
    },
    BackendType.SELENE: {
        "admin": False,
        "device": True,
        "dataset": True,
        "metrics": True,
        "wolfram": True,
        "geolocate": True,
        "stt": True,
        "owm": True,
        "email": True,  # only send to email used for registering account
        "oauth": True
    },
    BackendType.PERSONAL: {
        "admin": True,
        "device": True,
        "dataset": True,
        "metrics": True,
        "wolfram": True,
        "geolocate": True,
        "stt": True,
        "owm": True,
        "email": True,
        "oauth": True  # can use local backend UI to register apps
    },
    BackendType.OVOS_API: {
        "admin": True,  # fake support -> cast to offline backend type
        "device": True,  # fake support -> cast to offline backend type
        "dataset": True,  # fake support -> cast to offline backend type
        "metrics": True,  # fake support -> cast to offline backend type
        "wolfram": True,
        "geolocate": True,
        "stt": True,
        "owm": True,
        "email": True,
        "oauth": True  # fake support -> cast to offline backend type
    }
}


def get_backend_type(conf=None):
    conf = conf or Configuration()
    if "server" in conf:
        conf = conf["server"]
    if conf.get("disabled"):
        return BackendType.OFFLINE
    if "backend_type" in conf:
        return conf["backend_type"]
    url = conf.get("url")
    if not url:
        return BackendType.OFFLINE
    if "api.openvoiceos.com" in url:
        return BackendType.OVOS_API
    elif "mycroft.ai" in url:
        return BackendType.SELENE
    return BackendType.PERSONAL


def get_backend_config(url=None, version="v1", identity_file=None, backend_type=None):
    config = Configuration()
    config_server = config.get("server") or {}
    if not url:
        url = config_server.get("url")
        version = config_server.get("version") or version
        backend_type = backend_type or get_backend_type(config)
    elif not backend_type:
        backend_type = get_backend_type({"url": url})

    if not url and backend_type:
        if backend_type == BackendType.SELENE:
            url = SELENE_API_URL
        elif backend_type == BackendType.OVOS_API:
            url = OVOS_API_URL
        elif backend_type == BackendType.PERSONAL:
            url = "http://0.0.0.0:6712"
        else:
            url = "http://127.0.0.1"

    return url, version, identity_file, backend_type



import base64
import json

from ovos_utils.security import encrypt, decrypt

from ovos_backend_client.api import DeviceApi


class SeleneCloud:
    def __init__(self, db_id="SeleneCloud", url=None, version="v1"):
        self.api = DeviceApi(url, version)
        self.db_id = db_id

    def add_entry(self, data_id, data):
        if not isinstance(data, str):
            data = json.dumps(data)
        display_name = f"{self.db_id}|{data_id}"
        meta = {
            "skill_gid": f"{data_id}|{self.db_id}",
            "display_name": display_name,
            "skillMetadata": {
                "sections": [
                    {
                        "name": display_name,
                        "fields": [
                            {"type": "label",
                             "label": "This is encoded data, DO NOT EDIT in browser"},
                            {"type": "checkbox",  # special flag for local backend
                             "name": "__shared_settings",  # not tied to devices
                             "value": "true"},
                            {"type": "text",
                             "name": "encoded_data",
                             "value": data}
                        ]
                    }
                ]
            }
        }

        return self.api.put_skill_settings_v1(meta)

    def get_entry(self, data_id):
        data_id = f"{data_id}|{self.db_id}"
        settings = self.api.get_skill_settings_v1()
        data = None
        for s in settings:
            if s['skill_gid'] == data_id:
                sections = s['skillMetadata']['sections']
                data = sections[0]["fields"][-1]["value"]
                break
        return data


class SecretSeleneCloud(SeleneCloud):
    def __init__(self, key, db_id="SecretSeleneCloud", url=None, version="v1"):
        self.key = key
        super().__init__(db_id, url, version)

    def add_entry(self, data_id, data):
        if not isinstance(data, str):
            data = json.dumps(data)

        ciphertext, tag, nonce = encrypt(self.key, data)
        # b64 strings for storage
        tag = base64.encodebytes(tag).decode("utf-8")
        nonce = base64.encodebytes(nonce).decode("utf-8")
        ciphertext = base64.encodebytes(ciphertext).decode("utf-8")

        data = {"tag": tag, "nonce": nonce, "ciphertext": ciphertext}
        return super().add_entry(data_id, data)

    def get_entry(self, data_id):
        data = super().get_entry(data_id)
        if data:
            data = json.loads(data)
            ciphertext = data["ciphertext"].encode("utf-8")
            tag = data["tag"].encode("utf-8")
            nonce = data["nonce"].encode("utf-8")

            ciphertext = base64.decodebytes(ciphertext)
            tag = base64.decodebytes(tag)
            nonce = base64.decodebytes(nonce)

            return decrypt(self.key, ciphertext, tag, nonce)


if __name__ == "__main__":
    cloud = SeleneCloud(url="http://0.0.0.0:6712")
    cloud.add_entry("test3", {"secret": "NOT ENCRYPTED MAN"})
    print(cloud.get_entry("test3"))

    k = "D8fmXEP5VqzVw2HE"
    cloud = SecretSeleneCloud(k, url="http://0.0.0.0:6712")
    cloud.add_entry("test7", {"secret": "secret data"})
    print(cloud.get_entry("test7"))
